#!/usr/bin/env python3
# -*- coding: UTF-8 -*-

"""
@Time    : 8/18/20 2:05 PM
@Author  : Rodney Cheung
@File    : time.py
@Software: PyCharm
"""

import time


def format_seconds(seconds, time_format):
    """
    convert seconds to specified format
    Args:
        seconds: seconds
        time_format: specified time format,hour minute & second
        can be used

    Returns:
        formatted time str
    """
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    return str(time_format).format(h, m, s)


def get_current_timestamp():
    """
    get current timestamp
    Returns:
        current timestamp
    """
    return time.time()


def now(time_format: str) -> str:
    return time.strftime(time_format, time.localtime())



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : thread_pool.py
# Time       ：2020/8/19 15:09
# Author     ：Rodney Cheung
"""

import contextlib
import queue
import threading

StopEvent = object()


class ThreadPool(object):
    def __init__(self, max_num, max_task_num=None):
        if max_task_num:
            self.q = queue.Queue(max_task_num)
        else:
            self.q = queue.Queue()
        self.max_num = max_num
        self.cancel = False
        self.terminal = False
        self.generate_list = []
        self.free_list = []

    def run(self, func, args, callback=None):
        """
        线程池执行一个任务
        :param func: 任务函数
        :param args: 任务函数所需参数
        :param callback: 任务执行失败或成功后执行的回调函数，回调函数有两个参数1、任务函数执行状态；2、任务函数返回值（默认为None，即：不执行回调函数）
        :return: 如果线程池已经终止，则返回True否则None
        """
        if self.cancel:
            return
        if len(self.free_list) == 0 and len(self.generate_list) < self.max_num:
            self.generate_thread()
        w = (
            func,
            args,
            callback,
        )
        self.q.put(w)

    def generate_thread(self):
        """
        创建一个线程
        """
        t = threading.Thread(target=self.call)
        t.start()

    def call(self):
        """
        循环去获取任务函数并执行任务函数
        """
        current_thread = threading.currentThread
        self.generate_list.append(current_thread)
        event = self.q.get()
        while event != StopEvent:
            func, arguments, callback = event
            try:
                result = func(*arguments)
                success = True
            except Exception:
                success = False
                result = None
            if callback is not None:
                try:
                    callback(success, result)
                except Exception:
                    pass
            with self.worker_state(self.free_list, current_thread):
                if self.terminal:
                    event = StopEvent
                else:
                    event = self.q.get()
        else:
            self.generate_list.remove(current_thread)

    def close(self):
        """
        执行完所有的任务后，所有线程停止
        """
        self.cancel = True
        full_size = len(self.generate_list)
        while full_size:
            self.q.put(StopEvent)
            full_size -= 1

    def terminate(self):
        """
        无论是否还有任务，终止线程
        """
        self.terminal = True
        while self.generate_list:
            self.q.put(StopEvent)
        self.q.empty()

    def is_busy(self):
        return len(self.generate_list) > 0

    @contextlib.contextmanager
    def worker_state(self, state_list, worker_thread):
        """
        用于记录线程中正在等待的线程数
        """
        state_list.append(worker_thread)
        try:
            yield
        finally:
            state_list.remove(worker_thread)



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : test_time_util.py
# Time       ：2020/8/12 16:32
# Author     ：Rodney Cheung
"""
import unittest
from wisbec.date import time


class TestTimeUtil(unittest.TestCase):
    def test_format_time_use(self):
        print(time.format_seconds(100, '{:d}h{:d}m{:d}s'))

    def test_get_current_timestamp(self):
        print(time.get_current_timestamp())


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : filesystem.py
# Time       ：2020/8/19 15:09
# Author     ：Rodney Cheung
"""
import os
import shutil


def is_directory_exist(dir_full_path) -> bool:
    """
    symbolic link or dir
    Args:
        dir_full_path: dir path

    Returns:

    """
    return os.path.isdir(dir_full_path)


def is_file_exist(file_full_path) -> bool:
    """
    symbolic link or file
    Args:
        file_full_path:file path

    Returns:

    """
    return os.path.isfile(file_full_path)


def create_directories(dir_full_path):
    """
    create directory recursively
    Args:
        dir_full_path: dir path

    Returns:

    """
    if not is_directory_exist(dir_full_path):
        os.makedirs(dir_full_path)


def remove(full_path):
    if not os.path.exists(full_path):
        return
    else:
        if os.path.isfile(full_path):
            os.remove(full_path)
        elif os.path.isdir(full_path):
            shutil.rmtree(full_path, ignore_errors=True)


def create_file(full_path):
    if os.path.exists(full_path):
        print("{0} already existed.".format(full_path))
        return

    f = open(full_path, 'w')
    f.close()


def copy_file(src_full_path, dst_full_path):
    shutil.copy2(src_full_path, dst_full_path)


def replace_extension(file_path, replacement):
    """
    replace file name's extension
    Args:
        file_path: file path
        replacement: target extension

    Returns:
        replaced file name
    """
    base_name, extension = os.path.splitext(file_path)
    if len(replacement) == 0:
        return base_name
    else:
        return base_name + '.' + replacement


def list_dir_recursively(path: str, include_ext_name=None) -> list:
    """
    list files on path
    Args:
        path: target path
        include_ext_name: include extensions,if None,all files are listed

    Returns:
        file lists
    """
    files = os.listdir(path)
    res = list()
    for file in files:
        filepath = os.path.join(path, file)
        if os.path.isdir(filepath):
            res.extend(list_dir_recursively(filepath, include_ext_name))
        else:
            if (include_ext_name is None) or (include_ext_name is not None and filepath.endswith(include_ext_name)):
                res.append(filepath)
    return res



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : test_zip.py
# Time       ：2020/8/19 15:54
# Author     ：Rodney Cheung
"""
import os
import unittest

from wisbec.file import zip


class TestZip(unittest.TestCase):
    def test_create_zip_file(self):
        zip.create_zip_file(os.path.join(self.test_data_dir, 'test.zip'),
                            os.path.join(os.path.pardir, 'wisbec'))


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : test_console_shell.py
# Time       ：2020/8/20 17:37
# Author     ：Rodney Cheung
"""
import unittest
from wisbec.console import shell


class TestConsoleShell(unittest.TestCase):
    def test_exec_cmd(self):
        code, output, err = shell.exec_cmd(['which', 'mitmdump'])
        print(code)
        print(output)
        print(err)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : const.py
# Time       ：2020/8/19 15:15
# Author     ：Rodney Cheung
"""
import sys


class Const(object):
    # pylint: disable=too-few-public-methods
    class ConstError(TypeError):
        pass

    class ConstCaseError(ConstError):
        pass

    def __setattr__(self, key, value):
        if key in self.__dict__:
            raise self.ConstError("Can't change const.{0}!".format(key))
        if not key.isupper():
            raise self.ConstCaseError(
                ("const name '{0}' is not all uppercase!".format(key)))
        self.__dict__[key] = value


sys.modules[__name__] = Const()



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : adb.py
# Time       ：2020/8/19 15:09
# Author     ：Rodney Cheung
"""

import time

from wisbec.console.shell import exec_cmd
from wisbec.logging.log import Log


class Adb(object):

    @staticmethod
    def devices(alive: bool = False) -> [str]:
        """
        获取所有设备列表
        :param alive: 只显示在线的设备
        :return: 设备号数组
        """
        devices = []
        ret_code, out, err = Adb.exec("devices")
        for line in out.split('\n')[1:]:
            splits = line.split()
            if len(splits) >= 2:
                device = splits[0]
                status = splits[1]
                if not alive or status == "device":
                    devices.append(device)
        return devices

    @staticmethod
    def wait_device(dev_id, timeout):
        s = time.time()
        while time.time() - s <= timeout:
            list_devs = Adb.devices(alive=True)
            if dev_id in list_devs:
                return True
            Log.warning('等待{}设备上线中...', dev_id)
            time.sleep(1)
        pass

    # return: code, out, err
    @staticmethod
    def exec(*args):
        """
        执行命令
        :param args: 命令
        :return: 输出结果
        """
        args = ['adb'] + list(args)
        code, out, err = exec_cmd(args)
        return code, out, err

    @staticmethod
    def shell(device_id, *args):
        code, out, err = Adb.exec('-s', device_id, 'shell', *args)
        return code, out, err

    @staticmethod
    def top_app(device_id):
        code, out, err = Adb.shell(device_id,
                                   "dumpsys", "activity", "top", "|", "grep", "^TASK", "-A", "1")
        try:
            return out.split('\n')[1].strip().split(' ')[1].split('/')[0]
        except:
            return None

    @staticmethod
    def screen_cap(device_id, save_path):
        code, out, err = Adb.shell(device_id,
                                   "screencap", "-p", save_path)
        return out == '' and err == ''

    @staticmethod
    def push(device_id, src, dst):
        code, out, err = Adb.exec('-s', device_id, 'push', src, dst)
        return code == 0

    @staticmethod
    def pull(device_id, src, dst):
        code, out, err = Adb.exec('-s', device_id, 'pull', src, dst)
        return code == 0

    @staticmethod
    def install(device_id, apk_path):
        code, out, err = Adb.exec('-s', device_id, 'install', apk_path)
        return 'Success' in out

    @staticmethod
    def id(device_id):
        code, out, err = Adb.exec('-s', device_id, 'id')
        s = out.find('uid=')
        e = out.find('(')
        return int(out[s + 4:e])

    # is_suc, msg
    @staticmethod
    def disable_verify(device_id):
        code, out, err = Adb.exec('-s', device_id, 'disable-verity')
        if err != '':
            return False, err
        return True, ''



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : zip.py
# Time       ：2020/8/19 15:50
# Author     ：Rodney Cheung
"""
import shutil


def create_zip_file(output_full_path, input_dir):
    shutil.make_archive(output_full_path, 'zip', input_dir)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : test_filesystem.py
# Time       ：2020/8/19 15:53
# Author     ：Rodney Cheung
"""
import os
import unittest

from wisbec.file import filesystem


class TestFilesystem(unittest.TestCase):

    @classmethod
    def setUpClass(cls) -> None:
        cls.test_data_dir = os.path.join(os.getcwd(), 'test_data')
        cls.test_filesystem_py = os.path.abspath(__file__)
        cls.test_dir = os.path.join(cls.test_data_dir, 'test_directory')
        cls.test_file = os.path.join(cls.test_data_dir, "test_file.txt")

    def test_filesystem(self):
        # test is_dir_exist
        self.assertEqual(filesystem.is_directory_exist('/test/wq/code/Future-Walkers'), False)
        self.assertEqual(filesystem.is_directory_exist(self.test_data_dir), True)
        # test is_file_exist
        self.assertEqual(filesystem.is_file_exist(self.test_filesystem_py), True)
        # test create_directory
        filesystem.create_directories(self.test_dir)
        self.assertEqual(filesystem.is_directory_exist(self.test_dir), True)
        # test remove directory
        filesystem.remove(self.test_dir)
        self.assertEqual(filesystem.is_directory_exist(self.test_dir), False)
        # test create_file
        filesystem.create_file(self.test_file)
        self.assertEqual(filesystem.is_file_exist(self.test_file), True)
        # test remove file
        filesystem.remove(self.test_file)
        self.assertEqual(filesystem.is_file_exist(self.test_file), False)

    def test_replace_extension(self):
        self.assertEqual(filesystem.replace_extension(os.path.join(os.getcwd(), 'requirements.txt'), ''),
                         os.path.join(os.getcwd(), 'requirements'))
        self.assertEqual(filesystem.replace_extension(os.path.join(os.getcwd(), 'requirements.txt'), 'png'),
                         os.path.join(os.getcwd(), 'requirements.png'))

    def test_list_dir_recursively(self):
        res = filesystem.list_dir_recursively('/Users/jsrdzhk/PycharmProjects', '.xml')
        print(res)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
@Time    : 8/18/20 2:01 PM
@Author  : Rodney Cheung
@File    : test_log.py
@Software: PyCharm
"""

import logging
import os
import unittest

from wisbec.logging.log import Log


class TestLog(unittest.TestCase):
    def setUp(self):
        Log.init_logger(os.path.join(os.getcwd(), 'log'))
        self.assertEqual(Log.is_log_to_file, True)
        self.assertEqual(Log.is_log_to_console, True)

    def tearDown(self):
        Log.close()
        self.assertEqual(len(Log.file_loggers), 0)

    def test_debug(self):
        log_content = 'test_debug{}{}'
        try:
            raise RuntimeWarning('runtime warning')
        except RuntimeWarning as e:
            Log.debug('catch exception: ', e)
        Log.debug(log_content, ' another', ' test_debug')

    def test_warning(self):
        log_content = 'test_warning{}{}'
        Log.warning(log_content, ' another', ' test_warning')

    def test_error(self):
        log_content = 'test_error{}{}'
        Log.error(log_content, ' another', ' test_error')

    def test_info(self):
        log_content = 'test_info{}{}'
        Log.info(log_content, ' another', ' test_info')

    def test_critical(self):
        placeholder = 'placeholder'
        Log.critical('{} test', placeholder)
        log_content = 'test_critical {} {}'
        another = 'another'
        test_critical = 'test_critical'
        Log.critical(log_content, another, test_critical)

    def test_set_console_log_level(self):
        Log.set_console_log_level(logging.ERROR)
        Log.warning('test_set_console_log_level')


if __name__ == '__main__':
    unittest.main()



# !/usr/bin/env python3
# -*-coding:utf-8 -*-

"""
# File       : test_file_reader.py
# Time       ：2020/8/19 15:53
# Author     ：Rodney Cheung
"""
import unittest

from wisbec.file import file_reader


class TestFile(unittest.TestCase):
    def test_get_file_md5(self):
        print(file_reader.get_file_md5('/home/rodneycheung/.mitmproxy/mitmproxy-ca-cert.pem'))
        print(file_reader.get_file_md5('/home/rodneycheung/workspace/Tweezer/tweezer/src/c8750f0d.0'))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
@Time    : 8/18/20 2:05 PM
@Author  : Rodney Cheung
@File    : file_reader.py
@Software: PyCharm
"""
import hashlib
import os
import typing


def get_file_md5(file_full_path):
    if not os.path.isfile(file_full_path):
        return
    md5hash = hashlib.md5()
    with open(file_full_path, 'rb') as f:
        while True:
            b = f.read(8096)
            if not b:
                break
            md5hash.update(b)

    return md5hash.hexdigest()


def read_file(path, bytes_num=-1) -> typing.AnyStr:
    with open(path, 'rb') as f:
        return f.read(bytes_num)


def last_line(file_path):
    with open(file_path) as fp:
        lines = fp.readlines()
        if len(lines) == 0:
            return ""
        return lines[-1].strip()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
# File       : setup.py
# Time       ：2020/8/19 15:53
# Author     ：Rodney Cheung
"""
from glob import glob
from os.path import basename
from os.path import splitext

import setuptools

with open('README.md', 'r') as f:
    long_description = f.read()

with open('LICENSE') as f:
    lic = f.read()

setuptools.setup(
    name='wisbec',
    version='0.1.1',
    author='rodneycheung',
    author_email='jsrdzhk@gmail.com',
    description='python wrappers',
    long_description='python wrappers',
    long_description_content_type='text/markdown',
    url='https://github.com/Future-Walkers/python-mod',
    license=lic,
    packages=setuptools.find_packages('src'),
    package_dir={'': 'src'},
    py_modules=[splitext(basename(path))[0] for path in glob('src/*.py')],
    platforms=['any'],
    install_requires=['sshtunnel', 'pymysql', 'colorlog'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: Unix',
        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Utilities',
    ],
    zip_safe=False)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
"""
@Author: Rodney Cheung
@Date: 2020-06-30 09:58:50
@LastEditors: Rodney Cheung
@LastEditTime: 2020-07-01 09:39:11
@FilePath: /python-wisbec/test/test_db_util.py
"""
from wisbec.database.db import MySqlDataBase
import unittest


class TestDbUtils(unittest.TestCase):
    def test_connect_by_ssh_tunnel(self):
        mysql_db = MySqlDataBase('sherry', 'sherry', '192.168.37.5', 3306, 'sherry',
                                True)
        mysql_db.connect_by_ssh_tunnel('192.168.49.59', 22, 'ubuntu',
                                      '/Users/rodneycheung/.ssh/id_rsa')
        query = 'SELECT count(TABLE_NAME) FROM information_schema.TABLES WHERE TABLE_SCHEMA=\'sherry\''
        with mysql_db.connection.cursor() as cursor:
            cursor.execute(query)
            query_res = cursor.fetchall()
            self.assertEqual(query_res, 8)

    def test_connect(self):
        mysql_db = MySqlDataBase('root', 'wq', '127.0.0.1', 3306, 'sherry')
        mysql_db.connect()
        query = 'SELECT count(TABLE_NAME) FROM information_schema.TABLES WHERE TABLE_SCHEMA=\'sherry\''
        with mysql_db.connection.cursor() as cursor:
            cursor.execute(query)
            query_res = cursor.fetchall()
            self.assertEqual(query_res, 8)
